@@ -207,11 +207,6 @@ feglm <- function(
   attr(X, "dimnames") <- NULL
   p <- ncol(X)
   
-  # Check for linear dependence in 'X'
-  if (qr(X)[["rank"]] < p) {
-    stop("Linear dependent terms detected.", call. = FALSE)
-  }
-  
   # Extract weights if required
   if (is.null(weights)) {
     wt <- rep(1.0, nt)
@@ -227,6 +222,11 @@ feglm <- function(
     stop("negative weights are not allowed.", call. = FALSE)
   }
   
+  # Check for linear dependence in 'X'
+  if (lm.wfit(X, y, wt)[["rank"]] < p) {
+    stop("Linear dependent terms detected.", call. = FALSE)
+  }
+  
   # Compute and check starting guesses
   if (!is.null(beta.start) || !is.null(eta.start)) {
     # If both are specified, ignore eta.start
