@@ -127,11 +127,6 @@ feglm.nb <- function(
   attr(X, "dimnames") <- NULL
   p <- ncol(X)
   
-  # Check for linear dependence in 'X'
-  if (qr(X)[["rank"]] < p) {
-    stop("Linear dependent terms detected.", call. = FALSE)
-  }
-  
   # Extract weights if required
   if (is.null(weights)) {
     wt <- rep(1.0, nt)
@@ -147,6 +142,11 @@ feglm.nb <- function(
     stop("negative weights are not allowed.", call. = FALSE)
   }
   
+  # Check for linear dependence in 'X'
+  if (lm.wfit(X, y, wt)[["rank"]] < p) {
+    stop("Linear dependent terms detected.", call. = FALSE)
+  }
+  
   # Check starting guess of \theta
   if (is.null(init.theta)) {
     family <- poisson(link)
