@@ -39,9 +39,9 @@ coef.feglm <- function(object, ...) {
 
 
 #' @title
-#' Extract coefficient matrix of average partial effects
+#' Extract coefficient matrix for average partial effects
 #' @description
-#' \code{\link{coef.summary.APEs}} is a generic function which extracts a coefficient matrix of 
+#' \code{\link{coef.summary.APEs}} is a generic function which extracts a coefficient matrix for 
 #' average partial effects from objects returned by \code{\link{getAPEs}}.
 #' @param 
 #' object an object of class \code{"summary.APEs"}.
@@ -59,9 +59,9 @@ coef.summary.APEs <- function(object, ...) {
 
 
 #' @title
-#' Extract coefficient matrix of structural parameters
+#' Extract coefficient matrix for structural parameters
 #' @description
-#' \code{\link{coef.summary.feglm}} is a generic function which extracts a coefficient matrix of 
+#' \code{\link{coef.summary.feglm}} is a generic function which extracts a coefficient matrix for 
 #' structural parameters from objects returned by \code{\link{feglm}}.
 #' @param 
 #' object an object of class \code{"summary.feglm"}.
@@ -165,9 +165,12 @@ print.APEs <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
 #' \code{\link{feglm}}
 #' @export
 print.feglm <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
-  cat(sub("\\(.*\\)", "", x[["family"]][["family"]]), " - ",
-      x[["family"]][["link"]], " link",
-      ", l= [", paste0(x[["lvls.k"]], collapse = ", "), "]\n\n", sep = "")
+  cat(
+    sub("\\(.*\\)", "", x[["family"]][["family"]]), " - ",
+    x[["family"]][["link"]], " link",
+    ", l= [", paste0(x[["lvls.k"]], collapse = ", "), "]\n\n",
+    sep = ""
+    )
   print(x[["coefficients"]], digits = digits)
 }
 
@@ -209,19 +212,27 @@ print.summary.APEs <- function(x, digits = max(3L, getOption("digits") - 3L), ..
 #' \code{\link{feglm}}
 #' @export
 print.summary.feglm <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
-  cat(sub("\\(.*\\)", "", x[["family"]][["family"]]), " - ",
-      x[["family"]][["link"]], " link\n\n", sep = "")
+  cat(
+    sub("\\(.*\\)", "", x[["family"]][["family"]]), " - ",
+    x[["family"]][["link"]], " link\n\n", sep = ""
+    )
   print(x[["formula"]])
   cat("\nEstimates:\n")
   printCoefmat(x[["cm"]], P.values = TRUE, has.Pvalue = TRUE, digits = digits)
-  cat("\nresidual deviance= ",
-      format(x[["deviance"]], digits = max(5L, digits + 1L), nsmall = 2L),
-      ",\n", sep = "")
-  cat("null deviance= ",
-      format(x[["null.deviance"]], digits = max(5L, digits + 1L), nsmall = 2L),
-      ",\n", sep = "")
-  cat("n= ", x[["nobs"]][["nobs"]],
-      ", l= [", paste0(x[["lvls.k"]], collapse = ", "), "]\n", sep = "")
+  cat(
+    "\nresidual deviance= ",
+    format(x[["deviance"]], digits = max(5L, digits + 1L), nsmall = 2L),
+    ",\n", sep = ""
+    )
+  cat(
+    "null deviance= ",
+    format(x[["null.deviance"]], digits = max(5L, digits + 1L), nsmall = 2L),
+    ",\n", sep = ""
+    )
+  cat(
+    "n= ", x[["nobs"]][["nobs"]],
+    ", l= [", paste0(x[["lvls.k"]], collapse = ", "), "]\n", sep = ""
+    )
   if (x[["nobs"]][["nobs.na"]] > 0L | x[["nobs"]][["nobs.pc"]] > 0L) {
     cat("\n")
     if (x[["nobs"]][["nobs.na"]] > 0L) {
@@ -236,11 +247,13 @@ print.summary.feglm <- function(x, digits = max(3L, getOption("digits") - 3L), .
   } else {
     cat("\nNumber of Fisher Scoring Iterations:", x[["iter"]])
     cat("\nNumber of Outer Iterations:", x[["iter.outer"]])
-    cat("\ntheta= ",
-        format(x[["theta"]], digits = digits, nsmall = 2L),
-        ", std. error= ",
-        format(attr(x[["theta"]], "SE"), digits = digits, nsmall = 2L),
-        "\n", sep = "")
+    cat(
+      "\ntheta= ",
+      format(x[["theta"]], digits = digits, nsmall = 2L),
+      ", std. error= ",
+      format(attr(x[["theta"]], "SE"), digits = digits, nsmall = 2L),
+      "\n", sep = ""
+      )
   }
 }
 
@@ -304,11 +317,13 @@ summary.APEs <- function(object, ...) {
 #' @seealso
 #' \code{\link{feglm}}
 #' @export
-summary.feglm <- function(object,
-                          type         = c("hessian", "outer.product", "sandwich", "clustered"),
-                          cluster      = NULL,
-                          cluster.vars = NULL,
-                          ...) {
+summary.feglm <- function(
+  object,
+  type         = c("hessian", "outer.product", "sandwich", "clustered"),
+  cluster      = NULL,
+  cluster.vars = NULL,
+  ...
+  ) {
   # 'cluster.vars' is deprecated
   if (!is.null(cluster.vars)) {
     warning("'cluster.vars' is deprecated; please use 'cluster' instead.", call. = FALSE)
@@ -328,14 +343,16 @@ summary.feglm <- function(object,
   colnames(cm) <- c("Estimate", "Std. error", "z value", "Pr(> |z|)")
   
   # Generate result list
-  res <- list(cm            = cm, 
-              deviance      = object[["deviance"]],
-              null.deviance = object[["null.deviance"]],
-              iter          = object[["iter"]],
-              nobs          = object[["nobs"]],
-              lvls.k        = object[["lvls.k"]],
-              formula       = object[["formula"]],
-              family        = object[["family"]])
+  res <- list(
+    cm            = cm,
+    deviance      = object[["deviance"]],
+    null.deviance = object[["null.deviance"]],
+    iter          = object[["iter"]],
+    nobs          = object[["nobs"]],
+    lvls.k        = object[["lvls.k"]],
+    formula       = object[["formula"]],
+    family        = object[["family"]]
+    )
   if (inherits(object, "feglm.nb")) {
     res[["theta"]] <- object[["theta"]]
     res[["iter.outer"]] <- object[["iter.outer"]]
@@ -347,18 +364,37 @@ summary.feglm <- function(object,
 
 
 #' @title
-#' Extract estimates of the covariance matrix
+#' Compute covariance matrix after estimating \code{APEs}
 #' @description
-#' \code{\link{vcov.feglm}} computes an estimate of the covariance matrix of the estimator of the
-#' structural parameters from objects returned by \code{\link{feglm}}. The estimate is obtained
-#' using the Hessian, the scores, or a combination of boths after convergence.
+#' \code{\link{vcov.APEs}} estimates the covariance matrix for the estimator of the
+#' average partial effects from objects returned by \code{\link{getAPEs}}.
+#' @param 
+#' object an object of class \code{"APEs"}.
+#' @param 
+#' ... other arguments.
+#' @return
+#' The function \code{\link{vcov.APEs}} returns a named matrix of covariance estimates.
+#' @seealso
+#' \code{\link{getAPEs}}
+#' @export
+vcov.APEs <- function(object, ...) {
+  object[["vcov"]]
+}
+
+
+#' @title
+#' Compute covariance matrix after fitting \code{feglm}
+#' @description
+#' \code{\link{vcov.feglm}} estimates the covariance matrix for the estimator of the
+#' structural parameters from objects returned by \code{\link{feglm}}. The covariance is computed
+#' from the Hessian, the scores, or a combination of both after convergence.
 #' @param 
 #' object an object of class \code{"feglm"}.
 #' @param
 #' type the type of covariance estimate required. \code{"hessian"} refers to the inverse
 #' of the negative expected Hessian after convergence and is the default option. 
 #' \code{"outer.product"} is the outer-product-of-the-gradient estimator, 
-#' \code{"sandwich"} is the sandwich estimator (sometimes also refered as robust estimator), 
+#' \code{"sandwich"} is the sandwich estimator (sometimes also referred as robust estimator), 
 #' and \code{"clustered"} computes a clustered covariance matrix given some cluster variables.
 #' @param
 #' cluster a symbolic description indicating the clustering of observations.
@@ -377,11 +413,13 @@ summary.feglm <- function(object,
 #' @seealso
 #' \code{\link{feglm}}
 #' @export
-vcov.feglm <- function(object,
-                       type         = c("hessian", "outer.product", "sandwich", "clustered"),
-                       cluster      = NULL,
-                       cluster.vars = NULL,
-                       ...) {
+vcov.feglm <- function(
+  object,
+  type         = c("hessian", "outer.product", "sandwich", "clustered"),
+  cluster      = NULL,
+  cluster.vars = NULL,
+  ...
+  ) {
   # Check validity of input argument 'type'
   type <- match.arg(type)
   
@@ -394,93 +432,102 @@ vcov.feglm <- function(object,
     cluster <- as.formula(paste0("~", paste0(cluster.vars, collapse = "+")))
   }
   
-  # Compute requested type of estimated covariance matrix
-  p <- length(object[["coefficients"]])
+  # Compute requested type of covariance matrix
+  H <- object[["Hessian"]]
+  p <- ncol(H)
   if (type == "hessian") {
     # Check if the Hessian is invertible and compute its inverse
-    R <- try(chol(object[["Hessian"]]), silent = TRUE)
-    if (inherits(R, "try-error")) {
-      V <- matrix(Inf, p, p)
-    } else {
-      V <- chol2inv(R)
-    }
-  } else if (type == "outer.product") {
-    # Check if the OPG is invertible and compute its inverse
-    R <- try(chol(crossprod(object[["Score"]])), silent = TRUE)
+    R <- try(chol(H), silent = TRUE)
     if (inherits(R, "try-error")) {
       V <- matrix(Inf, p, p)
     } else {
       V <- chol2inv(R)
     }
   } else {
-    # Check if the Hessian is invertible and compute its inverse
-    R <- try(chol(object[["Hessian"]]), silent = TRUE)
-    if (inherits(R, "try-error")) {
-      V <- matrix(Inf, p, p)
-    } else {
-      # Extract data and the score
-      data <- object[["data"]]
-      G <- object[["Score"]]
-      
-      # Compute the inverse of the empirical Hessian
-      A <- chol2inv(R)
-      
-      # Compute inner part of the sandwich formula
-      if (type == "sandwich") {
-        B <- crossprod(G)
+    G <- getScoreMatrix(object)
+    if (type == "outer.product") {
+      # Check if the OPG is invertible and compute its inverse
+      R <- try(chol(crossprod(G)), silent = TRUE)
+      if (inherits(R, "try-error")) {
+        V <- matrix(Inf, p, p)
       } else {
-        # Check validity of input argument 'cluster'
-        if (is.null(cluster)) {
-          stop("'cluster' has to be specified.", call. = FALSE)
-        } else if (!inherits(cluster, "formula")) {
-          stop("'cluster' has to be of class formula.", call. = FALSE)
-        }
+        V <- chol2inv(R)
+      }
+    } else {
+      # Check if the Hessian is invertible and compute its inverse
+      R <- try(chol(H), silent = TRUE)
+      if (inherits(R, "try-error")) {
+        V <- matrix(Inf, p, p)
+      } else {
+        # Extract data
+        data <- object[["data"]]
         
-        # Extract cluster variables
-        cluster <- Formula(cluster)
-        D <- try(data[, all.vars(cluster), with = FALSE], silent = TRUE)
-        if (inherits(D, "try-error")) {
-          stop(paste("At least one cluster variable was not found.",
-                     "Ensure to pass variables that are not part of the model itself, but are", 
-                     "required to compute clustered standard errors, to 'feglm'.", 
-                     "This can be done via 'formula'. See documentation for details."),
-               call. = FALSE)
-        }
+        # Compute the inverse of the empirical Hessian
+        A <- chol2inv(R)
         
-        # Ensure cluster variables are factors
-        cl.vars <- names(D)
-        D[, (cl.vars) := lapply(.SD, checkFactor)]
-        
-        # Join cluster variables and scores
-        sp.vars <- colnames(G)
-        G <- cbind(D, G)
-        rm(D)
-        
-        # Multiway clustering ala Cameron, Gelbach, and Miller (2011)
-        setkeyv(G, cl.vars)
-        B <- matrix(0.0, p, p)
-        for (i in seq.int(length(cl.vars))) {
-          # Compute outer product for all possible combinations
-          cl.combn <- combn(cl.vars, i)
-          B.r <- matrix(0.0, p, p)
-          for (j in seq.int(ncol(cl.combn))) {
-            cl <- cl.combn[, j]
-            B.r <- B.r + 
-              crossprod(as.matrix(
-                G[, lapply(.SD, sum), by = cl, .SDcols = sp.vars][, sp.vars, with = FALSE]))
+        # Compute inner part of the sandwich formula
+        if (type == "sandwich") {
+          B <- crossprod(G)
+        } else {
+          # Check validity of input argument 'cluster'
+          if (is.null(cluster)) {
+            stop("'cluster' has to be specified.", call. = FALSE)
+          } else if (!inherits(cluster, "formula")) {
+            stop("'cluster' has to be of class formula.", call. = FALSE)
           }
           
-          # Update outer product
-          if (i %% 2L) {
-            B <- B + B.r
-          } else {
-            B <- B - B.r
+          # Extract cluster variables
+          cluster <- Formula(cluster)
+          D <- try(data[, all.vars(cluster), with = FALSE], silent = TRUE)
+          if (inherits(D, "try-error")) {
+            stop(
+              paste(
+                "At least one cluster variable was not found.",
+                "Ensure to pass variables that are not part of the model itself, but are", 
+                "required to compute clustered standard errors, to 'feglm'.", 
+                "This can be done via 'formula'. See documentation for details."
+              ),
+              call. = FALSE
+            )
+          }
+          
+          # Ensure cluster variables are factors
+          cl.vars <- names(D)
+          D[, (cl.vars) := lapply(.SD, checkFactor)]
+          
+          # Join cluster variables and scores
+          sp.vars <- colnames(G)
+          G <- cbind(D, G)
+          rm(D)
+          
+          # Multiway clustering by Cameron, Gelbach, and Miller (2011)
+          setkeyv(G, cl.vars)
+          B <- matrix(0.0, p, p)
+          for (i in seq.int(length(cl.vars))) {
+            # Compute outer product for all possible combinations
+            cl.combn <- combn(cl.vars, i)
+            B.r <- matrix(0.0, p, p)
+            for (j in seq.int(ncol(cl.combn))) {
+              cl <- cl.combn[, j]
+              B.r <- B.r + crossprod(
+                as.matrix(
+                  G[, lapply(.SD, sum), by = cl, .SDcols = sp.vars][, sp.vars, with = FALSE]
+                )
+              )
+            }
+            
+            # Update outer product
+            if (i %% 2L) {
+              B <- B + B.r
+            } else {
+              B <- B - B.r
+            }
           }
         }
+        
+        # Sandwich formula
+        V <- A %*% B %*% A
       }
-      
-      # Sandwich formula
-      V <- A %*% B %*% A
     }
   }
   
