@@ -16,21 +16,15 @@ feglmFit <- function(beta, eta, y, X, wt, k.list, family, control) {
   # Extract control arguments
   center.tol <- control[["center.tol"]]
   dev.tol <- control[["dev.tol"]]
-  epsilon <- max(min(1.0e-07, dev.tol / 1000.0), .Machine[["double.eps"]])
   iter.max <- control[["iter.max"]]
   trace <- control[["trace"]]
   keep.mx <- control[["keep.mx"]]
-  
+
   # Compute initial quantities for the maximization routine
-  nt <- length(y)
   mu <- family[["linkinv"]](eta)
   dev <- sum(family[["dev.resids"]](y, mu, wt))
   null.dev <- sum(family[["dev.resids"]](y, mean(y), wt))
-  
-  # Generate temporary variables
-  Mnu <- as.matrix(numeric(nt))
-  MX <- X
-  
+
   # Start maximization of the log-likelihood
   conv <- FALSE
   for (iter in seq.int(iter.max)) {
@@ -38,21 +32,21 @@ feglmFit <- function(beta, eta, y, X, wt, k.list, family, control) {
     eta.old <- eta
     beta.old <- beta
     dev.old <- dev
-    
+
     # Compute weights and dependent variable
     mu.eta <- family[["mu.eta"]](eta)
     w <- (wt * mu.eta^2) / family[["variance"]](mu)
-    w.tilde <- sqrt(w)
     nu <- (y - mu) / mu.eta
-    
+
     # Centering variables
-    Mnu <- centerVariables((Mnu + nu), w, k.list, center.tol)
-    MX <- centerVariables(MX, w, k.list, center.tol)
-    
+    Mnu <- as.vector(centerVariables(as.matrix(nu), w, k.list, center.tol))
+    MX <- centerVariables(X, w, k.list, center.tol)
+
     # Compute update step and update \eta
-    beta.upd <- as.vector(qr.solve(MX * w.tilde, Mnu * w.tilde, epsilon))
-    eta.upd <- nu - as.vector(Mnu - MX %*% beta.upd)
-    
+    wls.list <- lm.wfit(MX, Mnu, w)
+    beta.upd <- wls.list[["coefficients"]]
+    eta.upd <- nu - wls.list[["residuals"]]
+
     # Step-halving with three checks
     # 1. finite deviance
     # 2. valid \eta and \mu
@@ -65,16 +59,16 @@ feglmFit <- function(beta, eta, y, X, wt, k.list, family, control) {
       dev <- sum(family[["dev.resids"]](y, mu, wt))
       dev.crit <- is.finite(dev)
       val.crit <- family[["valideta"]](eta) && family[["validmu"]](mu)
-      imp.crit <- (dev - dev.old) / (0.1 + abs(dev)) <= - dev.tol
+      imp.crit <- (dev - dev.old) / (0.1 + abs(dev)) <= -dev.tol
       if (dev.crit && val.crit && imp.crit) break
       rho <- rho / 2.0
     }
-    
+
     # Check if step-halving failed (deviance and invalid \eta or \mu)
     if (!dev.crit || !val.crit) {
       stop("Inner loop failed; cannot correct step size.", call. = FALSE)
     }
-    
+
     # Stop if we do not improve
     if (!imp.crit) {
       eta <- eta.old
@@ -82,13 +76,13 @@ feglmFit <- function(beta, eta, y, X, wt, k.list, family, control) {
       dev <- dev.old
       mu <- family[["linkinv"]](eta)
     }
-    
+
     # Progress information
     if (trace) {
       cat("Deviance=", format(dev, digits = 5L, nsmall = 2L), "Iterations -", iter, "\n")
       cat("Estimates=", format(beta, digits = 3L, nsmall = 2L), "\n")
     }
-    
+
     # Check convergence
     dev.crit <- abs(dev - dev.old) / (0.1 + abs(dev))
     if (trace) cat("Stopping criterion=", dev.crit, "\n")
@@ -97,24 +91,21 @@ feglmFit <- function(beta, eta, y, X, wt, k.list, family, control) {
       conv <- TRUE
       break
     }
-    
-    # Update starting guesses for acceleration
-    Mnu <- Mnu - nu
   }
-  
+
   # Information if convergence failed
-  if (!conv && trace) cat("Algorithm did not converge.\n") 
-  
+  if (!conv && trace) cat("Algorithm did not converge.\n")
+
   # Update weights and dependent variable
   mu.eta <- family[["mu.eta"]](eta)
   w <- (wt * mu.eta^2) / family[["variance"]](mu)
-  
+
   # Center variables
   MX <- centerVariables(X, w, k.list, center.tol)
-  
+
   # Recompute Hessian
-  H <- crossprod(MX * sqrt(w))
-  
+  H <- crossprod(MX * w, MX)
+
   # Generate result list
   reslist <- list(
     coefficients  = beta,
@@ -125,11 +116,11 @@ feglmFit <- function(beta, eta, y, X, wt, k.list, family, control) {
     null.deviance = null.dev,
     conv          = conv,
     iter          = iter
-    )
-  
+  )
+
   # Update result list
   if (keep.mx) reslist[["MX"]] <- MX
-  
+
   # Return result list
   reslist
 }
@@ -151,18 +142,18 @@ feglmOffset <- function(object, offset) {
   lvls.k <- object[["lvls.k"]]
   nt <- object[["nobs"]][["nobs"]]
   k.vars <- names(lvls.k)
-  
+
   # Extract dependent variable
   y <- data[[1L]]
-  
+
   # Extract control arguments
   center.tol <- control[["center.tol"]]
   dev.tol <- control[["dev.tol"]]
   iter.max <- control[["iter.max"]]
-  
+
   # Generate auxiliary list of indexes to project out the fixed effects
   k.list <- getIndexList(k.vars, data)
-  
+
   # Compute starting guess for \eta
   if (family[["family"]] == "binomial") {
     eta <- rep(family[["linkfun"]](sum(wt * (y + 0.5) / 2.0) / sum(wt)), nt)
@@ -178,7 +169,6 @@ feglmOffset <- function(object, offset) {
   # Compute initial quantities for the maximization routine
   mu <- family[["linkinv"]](eta)
   dev <- sum(family[["dev.resids"]](y, mu, wt))
-  Myadj <- as.matrix(numeric(nt))
 
   # Start maximization of the log-likelihood
   for (iter in seq.int(iter.max)) {
@@ -189,11 +179,11 @@ feglmOffset <- function(object, offset) {
     # Compute weights and dependent variable
     mu.eta <- family[["mu.eta"]](eta)
     w <- (wt * mu.eta^2) / family[["variance"]](mu)
-    yadj <- (y - mu) / mu.eta + eta - offset
+    z <- (y - mu) / mu.eta + eta - offset
 
     # Centering dependent variable and compute \eta update
-    Myadj <- centerVariables((Myadj + yadj), w, k.list, center.tol)
-    eta.upd <- yadj - as.vector(Myadj) + offset - eta
+    Mz <- as.vector(centerVariables(as.matrix(z), w, k.list, center.tol))
+    eta.upd <- z - Mz + offset - eta
 
     # Step-halving with three checks
     # 1. finite deviance
@@ -206,7 +196,7 @@ feglmOffset <- function(object, offset) {
       dev <- sum(family[["dev.resids"]](y, mu, wt))
       dev.crit <- is.finite(dev)
       val.crit <- family[["valideta"]](eta) && family[["validmu"]](mu)
-      imp.crit <- (dev - dev.old) / (0.1 + abs(dev)) <= - dev.tol
+      imp.crit <- (dev - dev.old) / (0.1 + abs(dev)) <= -dev.tol
       if (dev.crit && val.crit && imp.crit) break
       rho <- rho / 2.0
     }
@@ -218,9 +208,6 @@ feglmOffset <- function(object, offset) {
 
     # Check termination condition
     if (abs(dev - dev.old) / (0.1 + abs(dev)) < dev.tol) break
-
-    # Update starting guesses for acceleration
-    Myadj <- Myadj - yadj
   }
 
   # Return \eta
@@ -245,14 +232,14 @@ getScoreMatrix <- function(object) {
   eta <- object[["eta"]]
   wt <- object[["weights"]]
   family <- object[["family"]]
-  
+
   # Update weights and dependent variable
   y <- data[[1L]]
   mu <- family[["linkinv"]](eta)
   mu.eta <- family[["mu.eta"]](eta)
   w <- (wt * mu.eta^2) / family[["variance"]](mu)
   nu <- (y - mu) / mu.eta
-  
+
   # Center regressor matrix (if required)
   if (control[["keep.mx"]]) {
     MX <- object[["MX"]]
@@ -260,20 +247,20 @@ getScoreMatrix <- function(object) {
     # Extract additional required quantities from result list
     formula <- object[["formula"]]
     k.vars <- names(object[["lvls.k"]])
-    
+
     # Generate auxiliary list of indexes to project out the fixed effects
     k.list <- getIndexList(k.vars, data)
-    
+
     # Extract regressor matrix
-    X <- model.matrix(formula, data, rhs = 1L)[, - 1L, drop = FALSE]
+    X <- model.matrix(formula, data, rhs = 1L)[, -1L, drop = FALSE]
     nms.sp <- attr(X, "dimnames")[[2L]]
     attr(X, "dimnames") <- NULL
-    
+
     # Center variables
     MX <- centerVariables(X, w, k.list, control[["center.tol"]])
     colnames(MX) <- nms.sp
   }
-  
+
   # Return score matrix
   MX * (nu * w)
 }
@@ -285,7 +272,7 @@ partialMuEta <- function(eta, family, order) {
   if (family[["link"]] != "logit") {
     eta <- family[["linkfun"]](family[["linkinv"]](eta))
   }
-  
+
   # Second- and third-order derivatives
   f <- family[["mu.eta"]](eta)
   if (order == 2L) {
@@ -293,11 +280,11 @@ partialMuEta <- function(eta, family, order) {
     if (family[["link"]] == "logit") {
       f * (1.0 - 2.0 * family[["linkinv"]](eta))
     } else if (family[["link"]] == "probit") {
-      - eta * f
+      -eta * f
     } else if (family[["link"]] == "cloglog") {
       f * (1.0 - exp(eta))
     } else {
-      - 2.0 * eta / (1.0 + eta^2) * f
+      -2.0 * eta / (1.0 + eta^2) * f
     }
   } else {
     # Third-order derivative
@@ -329,4 +316,4 @@ tempVar <- function(data) {
 # Unload
 .onUnload <- function(libpath) {
   library.dynam.unload("alpaca", libpath)
-}
\ No newline at end of file
+}
